package com.metro.aop;


import com.metro.baseRe.BaseResponse;
import com.metro.entity.error.CommonRes;
import com.metro.entity.error.ErrorMsg;
import com.metro.exception.BusinessException;
import com.metro.exception.OpenCommomException;
import org.assertj.core.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@RestControllerAdvice
public class GlobalExceptionHandler {

  private static Logger log = LoggerFactory.getLogger(GlobalExceptionHandler.class);

  @ResponseStatus(HttpStatus.OK)
  @ExceptionHandler(BusinessException.class)
  public BaseResponse handleEagleException(HttpServletRequest request, BusinessException exception) {

    if (exception.getCause() != null) {
      log.info("Request: " + request.getRequestURI() + " raised " + exception.getMessage(), exception.getCause());
    }
    return BaseResponse.fail(exception.getMessage());
  }

    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler(OpenCommomException.class)
    public BaseResponse handleEagleException(HttpServletRequest request, OpenCommomException exception) {

        if (exception.getCause() != null) {
            log.info("Request: " + request.getRequestURI() + " raised " + exception.getMessage(), exception.getCause());
        }
        return BaseResponse.fail(exception.getMessage());
    }


  @ResponseStatus(HttpStatus.OK)
  @ExceptionHandler(Exception.class)
  public BaseResponse handleGlobalException(HttpServletRequest request, Exception exception) {
    exception.printStackTrace();
    log.info("=======服务异常，e:{}",exception.getMessage());
    return BaseResponse.fail(exception.getMessage());
  }

  @ExceptionHandler({MethodArgumentNotValidException.class, BindException.class})
  public CommonRes exception(MethodArgumentNotValidException e) {
    BindingResult bindingResult = e.getBindingResult();
    List<ObjectError> allErrors = bindingResult.getAllErrors();
    List<ErrorMsg> errorMsgs = Lists.newArrayList();
    allErrors.forEach(objectError -> {
      ErrorMsg errorMsg = new ErrorMsg();
      FieldError fieldError = (FieldError) objectError;
      errorMsg.setField(fieldError.getField());
      errorMsg.setObjectName(fieldError.getObjectName());
      errorMsg.setMessage(fieldError.getDefaultMessage());
      errorMsgs.add(errorMsg);
    });
    Map<String, Object> result = new HashMap<>();
    result.put("errParam", errorMsgs);
    return CommonRes.builder().code(400).msg("参数错误").result(result).build();
  }

}
