package com.metro.baseRe;

import com.metro.entity.error.MsgCode;
import lombok.Data;

/**
 * 通用响应实体
 * @author lws
 */
@Data
public class BaseResponse<T> {

    /**
     * 状态码
     * 0成功，1失败
     */
    private Integer code;
    /**
     * 描述信息
     * 成功，或具体错误信息
     */
    private String msg;
    /**
     * 业务数据
     */
    private T data;

    private BaseResponse() {
    }

    private BaseResponse(Integer code, String msg, T data) {
        this.code = code;
        this.msg = msg;
        this.data = data;
    }

    public static <T> BaseResponse<T> ok() {
        return new BaseResponse<>(MsgCode.SUCCESS.code, MsgCode.SUCCESS.desc, null);
    }

    public static <T> BaseResponse<T> ok(T t) {
        return new BaseResponse<>(MsgCode.SUCCESS.code, MsgCode.SUCCESS.desc, t);
    }

    public static <T> BaseResponse<T> ok(String msg) {
        return new BaseResponse<T>(MsgCode.SUCCESS.code, msg , null);
    }

    public static <T> BaseResponse<T> fail(String msg) {
        return new BaseResponse<T>(MsgCode.FAILED.code, msg, null);
    }

    public static <T> BaseResponse<T> build(Integer code, String msg, T t) {
        return new BaseResponse<>(code, msg, t);
    }

    public static <T> BaseResponse<T> success(T data){
        BaseResponse<T> result=new BaseResponse<T>();
        result.setData(data);
        result.setCode(200);
        result.setMsg("成功");
        return result;
    }


}
