package com.metro.controller;


import cn.hutool.core.codec.Base64;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.metro.baseRe.BaseResponse;
import com.metro.entity.dto.JpersonDTO;
import com.metro.entity.dto.JpersonEditDTO;
import com.metro.entity.search.JPersonSearch;
import com.metro.entity.vo.JPersonVO;
import com.metro.restful.RestResponse;
import com.metro.restful.RestResult;
import com.metro.security.Authority;
import com.metro.service.JPersonsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.coobird.thumbnailator.Thumbnails;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import javax.validation.Valid;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author wxg
 * @since 2022-04-29
 */
@RestController
@Api(tags = "登录管理")
@RequestMapping("/jPersons")
public class JPersonsController {


    @Autowired
    private JPersonsService jPersonsService;
    /**
     * 登录
     * @return 是否成功
     */
    @ApiOperation("后台管理登录")
    @PostMapping("/login")
    @Authority(required = false)
    public BaseResponse<JPersonVO> doLogin(@Valid @RequestBody JpersonDTO jpersonDTO) {
        return BaseResponse.ok(jPersonsService.login(jpersonDTO));
    }

    @ApiOperation("后台管理退出登录")
    @Authority(required = false)
    @GetMapping("/doLogout")
    public RestResult<Boolean> doLogout() {
        return RestResponse.ok(jPersonsService.logout());
    }

    /**
     * 获取当前用户信息
     */
    @ApiOperation("获取用户信息")
    @GetMapping("/getUserInfo")
    public RestResult<JPersonVO> getUserInfo() {
        return RestResponse.ok(jPersonsService.getUserInfo());
    }

    /**
     * 账号列表查询
     */
    @ApiOperation("账号列表查询")
    @PostMapping("/getPageList")
    @Authority
    public BaseResponse<IPage<JPersonVO>> getPageList(@Valid @RequestBody(required = false) JPersonSearch jPersonSearch) {
        return BaseResponse.ok(jPersonsService.getPageList(jPersonSearch));
    }
    /**
     * 添加账号
     */
    @PostMapping("/insertOrUpdate")
    @Authority
    @ApiOperation("添加修改账号")
    public BaseResponse<Boolean> add(@Valid @RequestBody JpersonEditDTO jpersonEditDTO) {
        Integer insert = jPersonsService.insert(jpersonEditDTO);
        if(insert == 1){
            return BaseResponse.ok("添加成功");
        }else{
            return BaseResponse.fail("添加失败");
        }
    }

     /**
     * 删除账号
     */
    @GetMapping("/delete")
    @Authority
    @ApiOperation("删除账号")
    public BaseResponse<Boolean> delete(@RequestParam("id")Integer id) {
        Integer integer = jPersonsService.delete(id);
        if(integer == 1){
            return BaseResponse.ok("添加成功");
        }else{
            return BaseResponse.fail("添加失败");
        }
    }
    //上传图片
    @ApiOperation(value = "单图片上传接口", notes = "单图片上传接口")
    @PostMapping(value = "/pictureLocal")
    public BaseResponse<Object> pictureLocal(@RequestParam MultipartFile file) throws IOException {
        InputStream inputStream = file.getInputStream();

        BufferedImage image = Thumbnails.of(inputStream).size(400, 300).asBufferedImage();//变为400*300,遵循原图比例缩或放到400*某个高度
        //输出流
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        ImageIO.write(image, "jpg", stream);
        Object base64 = Base64.encode(stream.toByteArray());

//        BASE64Encoder base64Encoder =new BASE64Encoder();
//        String base64EncoderImg = file.getOriginalFilename()+","+ base64Encoder.encode(file.getBytes());
//        Object substring = base64EncoderImg.substring(base64EncoderImg.indexOf(",")+1);
        return BaseResponse.ok(base64);
    }
}
