package com.metro.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.metro.baseRe.BaseResponse;
import com.metro.entity.ro.passrecords.SelectPassreCordsPageListReq;
import com.metro.entity.ro.passrecords.SelectPassreCordsPageListResp;
import com.metro.entity.search.WarningSearch;
import com.metro.entity.vo.WarningDetailOneVO;
import com.metro.entity.vo.WarningVO;
import com.metro.service.PassreCordsService;
import com.metro.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @ClassName: PassreCordsController
 * @Description: TODO
 * @Author: ZS
 * @CreateName: lws
 * @Date 2022/3/8 12:13
 * ...
 */
@RestController
@RequestMapping("/passRecords")
@Api(tags = "盘查")
public class PassreCordsController {

    @Autowired
    private PassreCordsService passreCordsservice;

    /**
     * 查询公司信息
     * @param req 查询条件
     * @return page列表
     */
    @PostMapping("/page")
    @ApiOperation("盘查记录")
    public BaseResponse<IPage<SelectPassreCordsPageListResp>> page(@Valid @RequestBody SelectPassreCordsPageListReq req) {
        return BaseResponse.ok(passreCordsservice.page(req));
    }

    @PostMapping("/warningPageList")
    @ApiOperation("预警人员列表")
    public BaseResponse<IPage<WarningVO>> warningPageList(@Valid @RequestBody WarningSearch warningSearch) {
        return BaseResponse.ok(passreCordsservice.warningPageList(warningSearch));
    }
    @PostMapping("/excelWarningPageList")
    @ApiOperation("excel导出预警人员列表")
    public void  excelWarningPageList(@Valid @RequestBody WarningSearch warningSearch,HttpServletResponse response) {
        passreCordsservice.excelWarningPageList(warningSearch,response);
    }
    @GetMapping("/warningDeatilList")
    @ApiOperation("预警详情列表")
    public BaseResponse<List<SelectPassreCordsPageListResp>> warningDeatilList(@RequestParam("cardId") String cardId,
                                                                               @RequestParam(value = "startTime",required = false) String startTime,
                                                                               @RequestParam(value = "endTime",required = false) String endTime) {
        /*因为有多个地方使用该接口，所以在不改变原有的基础，添加新的查询条件就写成这样的，没有进行实体类封装*/
        SelectPassreCordsPageListReq req = new SelectPassreCordsPageListReq();
        req.setCardId(cardId);
        req.setStartTime(StringUtils.isNotEmpty(startTime) ? startTime:null);
        req.setEndTime(StringUtils.isNotEmpty(endTime) ? endTime:null);
        return BaseResponse.ok(passreCordsservice.warningDeatilList(req));
    }
    @GetMapping("/warningDeatil")
    @ApiOperation("单个预警详细信息")
    public BaseResponse<WarningDetailOneVO> warningDeatil(@RequestParam("id") Integer id) {
        return BaseResponse.ok(passreCordsservice.warningDeatil(id));
    }


//    @PostMapping("/delete")
//    @ApiOperation("删除盘查记录")
//    public BaseResponse<DeletePassreCordsResq> delete(@Valid @RequestBody DeletePassreCordsReq req) {
//        return BaseResponse.ok(passreCordsservice.delete(req));
//    }
//
//    @PostMapping("/selectById")
//    @ApiOperation("查看详情")
//    public BaseResponse<DetialPassreCordsResq> selectById(@Valid @RequestBody DetailPassreCordsReq req) {
//        return BaseResponse.ok(passreCordsservice.selectById(req));
//    }


}
