package com.metro.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.metro.baseRe.BaseResponse;
import com.metro.entity.model.UserOperateModel;
import com.metro.entity.ro.account.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @ClassName: AccountService
 * @Description: TODO
 * @Author: ZS
 * @CreateName: lws
 * @Date 2022/3/9 11:10
 * ...
 */
public interface AccountService {

    /**
     * 账号列表查询
     *
     * @param req 查询条件
     * @return page列表
     */
    IPage<SelectAccountPageResp> page(SelectAccountPageReq req);

    /**
     * 添加
     *
     * @param req 账号信息
     * @return 影响行数
     * 说明：
     *  每一级账号只能创建他下面一级的账号
     *  type = 1  创建 type = 2的账号以此类推
     *  type = 2 有companyId
     *  type 3 4 拥有companyId 和branchId
     *
     *
     *
     */
    InsertAccountResp add(InsertAccountReq req);

    /**
     * 校验账号是否存在
     *
     * @param req 账号
     * @return true/false
     */
    ValidAccountResp validAccount(ValidAccountReq req);

    /**
     * 删除账号
     *
     * @param req 主键
     * @return 影响行数
     */
    DeleteAccountResp delete(DeleteAccountReq req);

    /**
     * 账号详情
     *
     * @param req 主键
     * @return 账号详情
     */
    DetailAccountResp detail(DetailAccountReq req);

    /**
     * 修改账号
     *
     * @param req 主键
     * @return 影响行数
     */
    UpdateAccountResp update(UpdateAccountReq req);

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 返回信息
     */
    BaseResponse<String> uploadPhoto(MultipartFile file);

    /**
     * 预览照片
     *
     * @param response response
     * @param fileUrl  fileUrl
     */
    void showPhoto(HttpServletResponse response, String fileUrl);

    /**
     * 导入excel 用户数据
     *
     * @param filePath filePath
     */
    void importSheet(String filePath);

    /**
     * 登录
     * @param req 账号密码
     * @return 是否成功
     */
    LoginAccountResp login(LoginAccountReq req);



    /**
     * 获取用户信息
     * @return 是否成功
     */
    LoginAccountResp getUserInfo(Long userId);

    String importUser(List<UserOperateModel> userList,String type);

    Boolean logout();
}
