package com.metro.service.base;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.metro.constant.UserConstant;
import com.metro.entity.po.Account;
import com.metro.exception.BusinessException;
import com.metro.mapper.AccountsMapper;
import com.metro.restful.RestCode;
import com.metro.security.JWTUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

/**
 * @author ：lws
 * @date ：2022/3/19 17:58
 */
public class BaseService {
    @Autowired
    private AccountsMapper accountsMapper;

    public Long getUserId() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (requestAttributes == null) {
            throw new BusinessException(RestCode.UNAUTHORIZED.getMsg());
        }
        HttpServletRequest request = requestAttributes.getRequest();
        return getUserId(request);
    }

    public Long getUserId(HttpServletRequest request) {
        String token = request.getHeader("Authorization");
        token = JWTUtils.getUserId(token);
        if (StringUtils.isEmpty(token)) {
            throw new BusinessException(RestCode.UNAUTHORIZED.getMsg());
        }
        return Long.valueOf(token);
    }

    public Account getAccountInfo() {
        Account account = accountsMapper.selectOne(new LambdaQueryWrapper<Account>().eq(Account::getId, getUserId()).eq(Account::getIsDelete, UserConstant.NOT_DELETE));
        if(ObjectUtils.isEmpty(account)) throw new BusinessException("获取用户信息失败！");
        return account;
    }

}
