package com.metro.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.metro.baseRe.BaseResponse;
import com.metro.constant.UserConstant;
import com.metro.entity.model.UserOperateModel;
import com.metro.entity.po.Account;
import com.metro.entity.po.Area;
import com.metro.entity.po.Company;
import com.metro.entity.ro.account.*;
import com.metro.exception.BusinessException;
import com.metro.mapper.AccountsMapper;
import com.metro.mapper.AreaMapper;
import com.metro.restful.ServiceException;
import com.metro.security.JWTUtils;
import com.metro.service.AccountService;
import com.metro.service.CompanyService;
import com.metro.service.base.BaseService;
import com.metro.utils.*;
import com.metro.utils.bean.BeanValidators;
import io.github.classgraph.utils.Join;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Validator;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName: AccountServiceImpl
 * @Description: TODO
 * @Author: ZS
 * @CreateName: lws
 * @Date 2022/3/9 11:11
 * ...
 */
@Service
@Log4j2
public class AccountServiceImpl extends BaseService implements AccountService {

    @Autowired
    private AccountsMapper accountsMapper;
    @Autowired
    private AreaMapper areaMapper;
    @Autowired
    protected Validator validator;
    @Autowired
    private CompanyService companyService;

    /**
     * 账号查询
     *
     * @param req 查询条件
     * @return 返回结果
     */
    @Override
    public IPage<SelectAccountPageResp> page(SelectAccountPageReq req) {
        IPage<SelectAccountPageResp> respPage = new Page<>(req.getPageNum(), req.getPageSize());
        IPage<Account> selectPage = accountsMapper.selectPage(new Page<>(req.getPageNum(), req.getPageSize()), pageWrapper(req));
        if (CollectionUtils.isEmpty(selectPage.getRecords())) return respPage;
        Set<Integer> areaIdSet = selectPage.getRecords().stream().map(Account::getAreaId).collect(Collectors.toSet());
        Map<Integer, String> areaMap = areaMapper.selectList(new LambdaQueryWrapper<Area>().in(Area::getId, areaIdSet)).stream().collect(Collectors.toMap(Area::getId, p ->
                Join.join("-", areaMapper.selectList(new LambdaQueryWrapper<Area>().in(Area::getId, StringUtils.split(p.getDistrictSqe(), ","))).stream().map(Area::getName).collect(Collectors.toList()))
        ));

        Map<String, Company> companyMap = companyService.getCompanyMapByList(selectPage.getRecords(), Account::getCompanyId);
        Map<String, Company> barnchMap = companyService.getBranchMapByList(selectPage.getRecords(),Account::getBranchId);


        /*数据填入返回*/
        return selectPage.convert(p -> MapperManager.mapper(p, SelectAccountPageResp.class, q -> {
            q.setAreaDesc(areaMap.get(p.getAreaId()));
            q.setCompanyName(companyMap.get(p.getCompanyId()).getCompanyName());
            q.setBranchName(barnchMap.get(p.getCompanyId()+p.getBranchId()).getBranchName());
            q.setCompanyName(companyMap.get(p.getCompanyId()) == null?"" : companyMap.get(p.getCompanyId()).getCompanyName());
            q.setBranchName(barnchMap.get(p.getCompanyId()+p.getBranchId()) == null ? "" : barnchMap.get(p.getCompanyId()+p.getBranchId()).getBranchName());
        }));
    }

    // page wrapper 查询条件
    private LambdaQueryWrapper<Account> pageWrapper(SelectAccountPageReq req) {
        Account accountInfo = getAccountInfo();
        LambdaQueryWrapper<Account> wrapper = new LambdaQueryWrapper<>();
        wrapper.like(StringUtils.isNotEmpty(req.getUserName()), Account::getUserName, req.getUserName());
        wrapper.eq(StringUtils.isNotEmpty(req.getPhone()), Account::getPhone, req.getPhone());
        wrapper.eq(Account::getIsDelete, UserConstant.NOT_DELETE);
        wrapper.orderByDesc(Account::getCreateTime);
        if(accountInfo.getType()==2){
            wrapper.gt(Account::getType, accountInfo.getType());
        } else wrapper.eq(Account::getType, accountInfo.getType()+1);

        return wrapper;
    }

    /**
     * 添加账号
     *
     * @param req 账号信息
     * @return 影响行数
     */
    @Override
    public InsertAccountResp add(InsertAccountReq req) {
        Account accountInfo = getAccountInfo();
        Integer type = accountInfo.getType();
        if(type == 3){if(ObjectUtil.isEmpty(req.getPoliceId()))throw new BusinessException("请输入警号");}
        else{req.setPoliceId(null);}
        if (!StringUtil.isChinese(req.getUserName())) throw new BusinessException(UserConstant.NOT_CHINESE_USERNAME);
        Account account = accountsMapper.selectOne(new LambdaQueryWrapper<Account>().eq(Account::getAccount, req.getAccount()).eq(Account::getIsDelete, UserConstant.NOT_DELETE));
        if (ObjectUtil.isNotEmpty(account)) throw new BusinessException(account.getAccount() + "已经存在！不能注册");
        return new InsertAccountResp().setCount(accountsMapper.insert(MapperManager.mapper(req, Account.class, p -> {
            p.setPassword(MD5UtilNew.string2MD5(req.getPassword()))
                    .setCreateTime(LocalDateTime.now())
                    .setType(type + 1)
                    .setCreateId(accountInfo.getId());
            if(type == 2){
                p.setCompanyId(accountInfo.getCompanyId());
            }
            if(type == 3){
                p.setCompanyId(accountInfo.getCompanyId());
                p.setBranchId(accountInfo.getBranchId());
            }

        })));
    }

    /**
     * 校验账号是否存在
     *
     * @param req 账号
     * @return true/false
     */
    @Override
    public ValidAccountResp validAccount(ValidAccountReq req) {
        return new ValidAccountResp().setFlag(ObjectUtil.isNotEmpty(accountsMapper.selectOne(new LambdaQueryWrapper<Account>()
                .eq(Account::getAccount, req.getAccount()).eq(Account::getIsDelete, UserConstant.NOT_DELETE))));
    }

    /**
     * 删除账号
     *
     * @param req 主键
     * @return 影响行数
     */
    @Override
    public DeleteAccountResp delete(DeleteAccountReq req) {
        return new DeleteAccountResp().setCount(accountsMapper.updateById(new Account().setId(req.getId())
                .setIsDelete(UserConstant.DELETE).setUpdateTime(LocalDateTime.now())));
    }

    /**
     * 账号详情
     *
     * @param req 主键
     * @return 账号详情
     */
    @Override
    public DetailAccountResp detail(DetailAccountReq req) {
        Account account = accountsMapper.selectById(req.getId());
        if(account.getAreaId() == null){
            return MapperManager.mapper(account, DetailAccountResp.class);
        }
        Area area = areaMapper.selectById(account.getAreaId());
        String[] split = area.getDistrictSqe().trim().split(",");
        String areaId = null;
        String cityId = null;
        String provinceId = null;
        List<String> strings = Arrays.asList(split);
        // .11.150.1275.
        if(strings !=null) {
            for (int i = 0; i < strings.size(); i++) {
                if (i == 0) {
                    provinceId = strings.get(i);
                } else if (i == 1) {
                    cityId = strings.get(i);
                } else if (i == 2) {
                    areaId = strings.get(i);
                }
            }
        }
        String finalAreaId = areaId;
        String finalCityId = cityId;
        String finalprovinceId = provinceId;
        return MapperManager.mapper(account, DetailAccountResp.class, p -> {
            p.setAreaId(finalAreaId);
            p.setCityId(finalCityId);
            p.setProvinceId(finalprovinceId);
        });
    }

    /**
     * 修改账号
     *
     * @param req 主键
     * @return 影响行数
     */
    @Override
    public UpdateAccountResp update(UpdateAccountReq req) {

        return getUpdateAccountResp(req, accountsMapper, req.getId());
    }

    static UpdateAccountResp getUpdateAccountResp(UpdateAccountReq req, AccountsMapper accountsMapper, Integer id) {
        if (!StringUtil.isChinese(req.getUserName())) throw new BusinessException(UserConstant.NOT_CHINESE_USERNAME);
        return new UpdateAccountResp().setCount(accountsMapper.updateById(MapperManager.mapper(req, Account.class, p -> p.setPassword(MD5UtilNew.string2MD5(req.getPassword()))
                .setUpdateTime(LocalDateTime.now()).setUpdateId(id))));
    }

    /**
     * 上传文件
     *
     * @param file 文件
     * @return 返回信息
     */
    @Override
    public BaseResponse<String> uploadPhoto(MultipartFile file) {

        return FileUtils.uploadPhoto(file);

    }

    /**
     * 预览照片
     *
     * @param response response
     * @param fileUrl  fileUrl
     */
    @Override
    public void showPhoto(HttpServletResponse response, String fileUrl) {
        if (StringUtils.isEmpty(fileUrl)) throw new BusinessException("请输入文件路径");
        File fileBase = new File(fileUrl);
        try {
            if (!fileBase.exists()) throw new BusinessException("文件不存在");
            ServletOutputStream outputStream = response.getOutputStream();
            FileInputStream inputStream = new FileInputStream(fileBase);
            //读取写出
            int len = 0;
            byte[] buffer = new byte[1024];
            while ((len = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, len);
                outputStream.flush();
            }

            outputStream.close();
            inputStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 导入excel 用户数据
     *
     * @param filePath filePath
     */
    @Override
    public void importSheet(String filePath) {
        List<Object> objects = ExcelUtil.readLessThan1000Row(filePath);
        // 过滤头信息
        List<Object> finalObjects = objects;
        List<DetailAccountResp> detailAccountResps = new ArrayList<>();
        for (int i = 2; i < finalObjects.size(); i++) {
            Object o = finalObjects.get(i);
            System.out.println(o);
        }

    }

    /**
     * 登录
     *
     * @param req 账号密码
     * @return 是否成功
     */
    @Override
    public LoginAccountResp login(LoginAccountReq req) {
        Account account =  accountsMapper.selectOne(new LambdaQueryWrapper<Account>().eq(Account::getAccount, req.getAccount()).eq(Account::getIsDelete, UserConstant.NOT_DELETE));
        if (ObjectUtil.isEmpty(account)) throw new BusinessException("账号不存在!请先注册");
        if (!MD5UtilNew.string2MD5(req.getPassword()).equals(account.getPassword()))
            throw new BusinessException("密码错误!请重新输入");
        return MapperManager.mapper(account, LoginAccountResp.class, p -> {
            p.setToken(JWTUtils.sign(account.getId().toString(), account.getPassword(), DateUtils.addDays(new Date(), 1)));
        });
    }


    /**
     * 获取用户信息
     *
     * @return 是否成功
     */
    @Override
    public LoginAccountResp getUserInfo(Long userId) {
        Account account = accountsMapper.selectOne(new LambdaQueryWrapper<Account>().eq(Account::getId, userId).eq(Account::getIsDelete, UserConstant.NOT_DELETE));
        if (ObjectUtil.isEmpty(account)) throw new BusinessException("获取用户信息失败");
        return MapperManager.mapper(account, LoginAccountResp.class);
    }

    /**
     * 导入用户数据
     *
     * @param userList 用户数据列表
     * @return 结果
     */
    @Override
    @Transactional
    public String importUser(List<UserOperateModel> userList, String type) {
        if (CollectionUtils.isEmpty(userList)) {
            throw new ServiceException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        for (UserOperateModel user : userList) {
            Account account = accountsMapper.selectOne(new LambdaQueryWrapper<Account>().eq(Account::getAccount, user.getAccount()).eq(Account::getIsDelete, UserConstant.NOT_DELETE));
            try {
                if (ObjectUtil.isNotEmpty(account)) { // 存在账号
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + user.getAccount() + " 已存在");
                } else { // 账号不存在新增数据
                    BeanValidators.validateWithException(validator, user);
                    successNum++;
                    Account mapper = MapperManager.mapper(user, Account.class, p -> {
                        p.setPassword(MD5UtilNew.string2MD5(user.getPassword())).setType(Integer.valueOf(type)).setCreateTime(LocalDateTime.now());
                    });
                    accountsMapper.insert(mapper);
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getAccount() + " 导入成功");
                }

            } catch (Exception e) {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + user.getAccount() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new BusinessException(failureMsg.toString());
        } else {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    /**
     * 退出登录
     *
     * @return 是否成功
     */
    @Override
    public Boolean logout() {
        return null;
    }

    public static ThreadLocal<Long> loginUserThreadLocal = new ThreadLocal<>();

}

