package com.metro.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.metro.entity.po.Company;
import com.metro.entity.ro.company.SelectCompanyPageReq;
import com.metro.entity.ro.company.SelectCompanyPageResp;
import com.metro.entity.vo.*;
import com.metro.mapper.CompanysMapper;
import com.metro.service.CompanyService;
import com.metro.service.JPersonsService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @ClassName: ConpanysServiceImpl
 * @Description: TODO
 * @Author: ZS
 * @CreateName: lws
 * @Date 2022/3/8 13:40
 * ...
 */
@Service
public class CompanyServiceImpl implements CompanyService {

    @Resource
    private CompanysMapper companysMapper;
    @Resource
    private JPersonsService jPersonsService;
    /**
     * 公安查询
     * @param req 查询条件
     * @return 返回结果
     */
    @Override
    public List<SelectCompanyPageResp> page(SelectCompanyPageReq req) {
        List<Company> companies = companysMapper.selectList(selectCompanysWrapper(req));
        if(CollectionUtils.isEmpty(companies)) return new ArrayList<>();
        return companies.stream().map(p -> BeanUtil.toBean(p,SelectCompanyPageResp.class)).collect(Collectors.toList());
    }

    // 根据id查公安列表wrapper
    private Wrapper<Company> selectCompanysWrapper(SelectCompanyPageReq req) {
        LambdaQueryWrapper<Company> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(req.getCompanyId()),Company::getBranchId,req.getCompanyId());
        return wrapper;
    }

    @Override
    public List<CompanyVO> getCompanyList() {
        JPersonVO userInfo = jPersonsService.getUserInfo();
        if(userInfo.getType() == 1){
            return companysMapper.getCompanyList();
        }
        else {
            List<CompanyVO> list = new ArrayList<>();
            CompanyVO companyVO = new CompanyVO();
            companyVO.setCompanyId(userInfo.getCompanyId());
            companyVO.setCompanyName(userInfo.getCompanyName());
            list.add(companyVO);
            return list;
        }

    }

    @Override
    public List<BranchVO> getBranchList(String companyId) {
        List<BranchVO> list = companysMapper.getBranchList(companyId);
        return list;
    }


    private Map<String, Company> getByCompanySet(Set<String> companySet) {
        List<Company> companyList = null;
        if(companySet.size() >0) companyList = companysMapper.selectByCompanySet(companySet);
        else{companyList = new ArrayList<>();}
        return companyList.stream().collect(Collectors.toMap(Company::getCompanyId, Function.identity()));
    }

    private Map<String, Company> getByBranchSet(Set<String> barnchSet) {
        List<Company> barnchList = null ;
        if(barnchSet.size() >0)barnchList = companysMapper.selectByBarnchSet(barnchSet);
        else {barnchList = new ArrayList<>();}
        return barnchList.stream().collect(Collectors.toMap(Company::getBranchId, Function.identity()));
    }

    @Override
    public <T> Map<String, Company> getCompanyMapByList(List<T> list, Function<T, String> function) {
        Set<String> companySet = list.stream().map(function).collect(Collectors.toSet());
        return this.getByCompanySet(companySet);
    }

    @Override
    public <T> Map<String, Company> getBranchMapByList(List<T> list, Function<T, String> function) {
        Set<String> barnchSet = list.stream().map(function).collect(Collectors.toSet());
        return this.getByBranchSet(barnchSet);
    }

    @Override
    public List<CompanyCompanyVO> getAll() {
        /*1.获取所有的派出所*/
        List<CompanyVO> companyList = companysMapper.getCompanyList();
        /*2.获取派出所对应的站点*/
        LambdaQueryWrapper<Company> companyLambdaQueryWrapper = new LambdaQueryWrapper<>();
        companyLambdaQueryWrapper.in(Company::getCompanyId,companyList.stream().map(CompanyVO::getCompanyId).collect(Collectors.toList()));
        List<Company> companys = companysMapper.selectList(companyLambdaQueryWrapper);
        Map<String,List<CompanyBranchVO>> map = this.getCompanyBranchVO(companys);
        List<CompanyCompanyVO> list = new ArrayList<>();
        for (CompanyVO companyVO : companyList) {
            CompanyCompanyVO companyCompanyVO = new CompanyCompanyVO();
            companyCompanyVO.setId(companyVO.getCompanyId());
            companyCompanyVO.setName(companyVO.getCompanyName());
            companyCompanyVO.setLevel(1);
            BeanUtils.copyProperties(companyVO,companyCompanyVO);
            if (map.containsKey(companyVO.getCompanyId())){
                companyCompanyVO.setList(map.get(companyVO.getCompanyId()));
            }
            list.add(companyCompanyVO);
        }


        return list;
    }

    private Map<String, List<CompanyBranchVO>> getCompanyBranchVO(List<Company> companys) {
        Map<String, List<CompanyBranchVO>> map = new HashMap<>();
        for (Company company : companys) {
            boolean b = map.containsKey(company.getCompanyId());
            if(b){
                if(StringUtils.isNotEmpty(company.getBranchId()) && StringUtils.isNotEmpty(company.getBranchName())){
                    List<CompanyBranchVO> list = map.get(company.getCompanyId());
                    CompanyBranchVO companyBranchVO = new CompanyBranchVO();
                    companyBranchVO.setId(company.getBranchId());
                    companyBranchVO.setName(company.getBranchName());
                    companyBranchVO.setLevel(2);
                    list.add(companyBranchVO);
                    map.put(company.getCompanyId(),list);
                }
            }else {
                if(StringUtils.isNotEmpty(company.getBranchId()) && StringUtils.isNotEmpty(company.getBranchName())){
                    List<CompanyBranchVO> list = new ArrayList<>();
                    CompanyBranchVO companyBranchVO = new CompanyBranchVO();
                    companyBranchVO.setId(company.getBranchId());
                    companyBranchVO.setName(company.getBranchName());
                    companyBranchVO.setLevel(2);
                    list.add(companyBranchVO);
                    map.put(company.getCompanyId(),list);
                }
            }
        }
        return map;
    }
}
