package com.metro.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.metro.entity.dto.CourtControlDTO;
import com.metro.entity.po.CourtControl;
import com.metro.entity.search.CourtControlSearch;
import com.metro.entity.vo.CourtControlVO;
import com.metro.exception.BusinessException;
import com.metro.mapper.CourtControlMapper;
import com.metro.service.CourtControlService;
import com.metro.utils.MapperManager;
import com.metro.utils.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * <p>
 * 法院协控 服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-07-19
 */
@Service
public class CourtControlServiceImpl implements CourtControlService {

    @Resource
    private CourtControlMapper courtControlMapper;

    @Override
    public IPage<CourtControlVO> getPageList(CourtControlSearch courtControlSearch) {
        Page<CourtControl> page = new Page<>(courtControlSearch.getPageNum(),courtControlSearch.getPageSize());
        IPage<CourtControl>  pageList =courtControlMapper.selectPage(page,this.pageWrapper(courtControlSearch));
        if(pageList.getRecords().size() == 0) return new Page<>();
        return pageList.convert(p -> MapperManager.mapper(p, CourtControlVO.class, q ->{}));
    }

    private LambdaQueryWrapper<CourtControl> pageWrapper(CourtControlSearch courtControlSearch) {
        LambdaQueryWrapper<CourtControl> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.like(StringUtils.isNotEmpty(courtControlSearch.getXm()),CourtControl::getXm,courtControlSearch.getXm());
        lambdaQueryWrapper.like(StringUtils.isNotEmpty(courtControlSearch.getSfzh()),CourtControl::getSfzh,courtControlSearch.getSfzh());
        return lambdaQueryWrapper;
    }

    @Override
    public Integer insertOrUpdate(CourtControlDTO courtControlDTO) {
        if(StringUtils.isEmpty(courtControlDTO.getSfzh())){
            throw new BusinessException("身份证号不能为空！");
        }
        CourtControl courtControl = courtControlMapper.selectById(courtControlDTO.getSfzh());
        if(courtControl == null){
            courtControl = new CourtControl();
            BeanUtils.copyProperties(courtControlDTO,courtControl);
            return courtControlMapper.insert(courtControl);
        }else {
            BeanUtils.copyProperties(courtControlDTO,courtControl);
            return courtControlMapper.updateById(courtControl);
        }
    }

    @Override
    public Integer delete(String sfzh) {
        return courtControlMapper.deleteById(sfzh);
    }
}
