package com.metro.service.impl;
/*
 * @Description
 * @Date 2022/5/17 11:10
 * @Author wxg
 * */

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.metro.baseRe.BaseResponse;
import com.metro.entity.dto.DataShowMiddlerDTO;
import com.metro.entity.po.Company;
import com.metro.entity.po.DataShow;
import com.metro.entity.po.JDevices;
import com.metro.entity.po.PassreCords;
import com.metro.entity.vo.*;
import com.metro.mapper.CompanysMapper;
import com.metro.mapper.DataShowMapper;
import com.metro.mapper.JDevicesMapper;
import com.metro.mapper.PassreCordsMapper;
import com.metro.restful.ServiceException;
import com.metro.service.CompanyService;
import com.metro.service.DataShowService;
import com.metro.service.JPersonsService;
import com.metro.service.PassreCordsService;
import com.metro.utils.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class DataShowServiceImpl implements DataShowService {

    @Autowired
    private JPersonsService jPersonsService;
    @Resource
    private DataShowMapper dataShowMapper;
    @Resource
    private PassreCordsMapper passreCordsMapper;
    @Resource
    private CompanysMapper companysMapper;
    @Resource
    private JDevicesMapper jDevicesMapper;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private PassreCordsService passreCordsService;
    @Override
    public BaseResponse<DataShowTopVO> selectDataShowTop() {
        /*
         * 实现说明
         * 总盘查人数：今日之前盘查人数 + 今日盘查人数
         * 总预警人数： 今日之前预警人数 +今日预警人数
         * 数据库中会存储几个数据
         * 定时任务每天12点启动汇总
         * 今日之前的总盘查人数
         * 今日之前的总预警人数
         * 今日之前的存疑人数
         * 今日之前的抓铺人数
         * */
        JPersonVO jPersonVO = this.rootTrue();
        Integer adminType = jPersonVO.getType();
        DataShowTopVO dataShowTopVO = new DataShowTopVO();
        Integer totalNumber = 0;//盘查总人数
        Integer totalWarningNumber = 0;//总预警人数 = 总存疑人数 + 总抓捕人数
        Integer totalArrestNumber = 0;//总抓捕人数
        Integer totalDoubtNumber = 0;//总存疑人数

        /*1.获取数据库中，今日0点之前的数据*/
        DataShow dataShow = null;
        Map<Integer, Integer> dataShowMap = null;//存储当前账号今日之前的数据
        if (adminType == 1) {
            dataShow = dataShowMapper.selectByCompanyIdGroup(null,null);
        } else if (adminType == 2) {
            dataShow = dataShowMapper.selectByCompanyIdGroup(jPersonVO.getCompanyId(),null);
        } else {
            throw new ServiceException("无权操作");
        }
        if(dataShow != null){
            totalNumber = dataShow.getNumber();
            totalArrestNumber = dataShow.getArrestNumber();
            totalDoubtNumber = dataShow.getDoubtNumber();
            totalWarningNumber = totalArrestNumber + totalDoubtNumber;
        }


        /*2.获取数据库中，今日0点之后的数据*/
        Integer todayNumber = 0;//今日盘查人数
        Integer todayWarningNumber = 0;//今日预警人数 = 今日存疑人数 + 今日抓捕人数
        Integer todayArrestNumber = 0;//今日抓捕人数
        Integer todayDoubtNumber = 0;//今日存疑人数

        if (adminType == 1) {
            dataShow = dataShowMapper.selectByCompanyIdGroup(null,1);
        } else if (adminType == 2) {
            dataShow = dataShowMapper.selectByCompanyIdGroup(jPersonVO.getCompanyId(),1);
        } else {
            throw new ServiceException("无权操作");
        }
        if(dataShow != null){
            todayNumber = dataShow.getNumber();
            todayArrestNumber = dataShow.getArrestNumber();
            todayDoubtNumber = dataShow.getDoubtNumber();
            todayWarningNumber = todayArrestNumber + todayDoubtNumber;
        }
        /*3.获取站点数量*/
        LambdaQueryWrapper<Company> companyWrapper = new LambdaQueryWrapper<>();
        companyWrapper.eq(StringUtils.isNotEmpty(jPersonVO.getCompanyId()), Company::getCompanyId, jPersonVO.getCompanyId());
        Integer totalSiteNumber = companysMapper.selectCount(companyWrapper);
        /*4.获取设备数量*/
        LambdaQueryWrapper<JDevices> jDevicesMapperWrapper = new LambdaQueryWrapper<>();
        jDevicesMapperWrapper.eq(StringUtils.isNotEmpty(jPersonVO.getCompanyId()), JDevices::getCompanyId, jPersonVO.getCompanyId());
        Integer totalDeviceNumber = jDevicesMapper.selectCount(jDevicesMapperWrapper);
        /*5.数据整合返回*/
        dataShowTopVO.setTotalNumber(totalNumber);
        dataShowTopVO.setTodayNumber(todayNumber);
        dataShowTopVO.setTotalWarningNumber(totalWarningNumber);
        dataShowTopVO.setTodayWarningNumber(todayWarningNumber);
        dataShowTopVO.setTotalDoubtNumber(totalDoubtNumber);
        dataShowTopVO.setTotalArrestNumber(totalArrestNumber);
        dataShowTopVO.setTodayDoubtNumber(todayDoubtNumber);
        dataShowTopVO.setTodayArrestNumber(todayArrestNumber);
        dataShowTopVO.setTotalSiteNumber(totalSiteNumber);
        dataShowTopVO.setTotalDeviceNumber(totalDeviceNumber);

        return BaseResponse.ok(dataShowTopVO);
    }

    @Override
    public BaseResponse<DataShowMiddleVO> selectDataShowMiddler(DataShowMiddlerDTO dataShowMiddlerDTO) {
        JPersonVO jPersonVO = this.rootTrue();
        List<DataShowBrandchVO> dataShowList = null;
        if(jPersonVO.getType() == 1){
            dataShowList = dataShowMapper.getDataShowListByCompanyIdAndBranchId(null,dataShowMiddlerDTO);
        }else if(jPersonVO.getType() == 2) {
            dataShowList = dataShowMapper.getDataShowListByCompanyIdAndBranchId(jPersonVO.getCompanyId(),dataShowMiddlerDTO);
        }else {
            throw new ServiceException("无权操作");
        }
        /*获取数据中companyMap 和barnchMap 暂时只要branchName*/
//        Map<String, Company> companyMap = companyService.getCompanyMapByList(dataShowList, DataShow::getCompanyId);
        Map<String, Company> barnchMap = companyService.getBranchMapByList(dataShowList,DataShowBrandchVO::getBranchId);
        List<String> siteList = new ArrayList<>();
        List<Integer> numberList = new ArrayList<>();
        for (DataShowBrandchVO dataShow : dataShowList) {
            if(dataShowMiddlerDTO.getPassType() == 1){
                numberList.add(dataShow.getNumber());
            }else {
                numberList.add(dataShow.getWarningNumber());
            }
            siteList.add(barnchMap.get(dataShow.getCompanyId()+dataShow.getBranchId()) == null ?
                    "" : barnchMap.get(dataShow.getCompanyId()+dataShow.getBranchId()).getBranchName());
        }
        DataShowMiddleVO dataShowMiddleVO = new DataShowMiddleVO();
        dataShowMiddleVO.setSiteList(siteList);
        dataShowMiddleVO.setNumberList(numberList);
        return BaseResponse.ok(dataShowMiddleVO);
    }

    @Override
    public BaseResponse<DataShowMiddleSiteVO> selectDataShowMiddlerSite(Integer id) {
        this.rootTrue();
        if(id == null){//传值为空，则获取一个默认的数据
            List<SiteVO> siteVOList = this.selectSiteList();
            if(siteVOList == null || siteVOList.size() <= 0)throw new ServiceException("该账号没有站点");
            SiteVO siteVO = siteVOList.get(0);
            id = siteVO.getId();
        }
        Company company = companysMapper.selectById(id);
        if(company == null )throw new ServiceException("传递id有误，没有查找到数据");

        List<PassreCordsGroupByTimeVO> list = passreCordsMapper.selectDataShowMiddlerSite(company);
        List<Integer> numberList = new ArrayList<>();

        for (int i = 0; i < 12; i++) {
            Integer number = 0;
            for (int j = 0; j < list.size(); j++) {
                PassreCordsGroupByTimeVO pco = list.get(j);
                String substring = pco.getDate().substring(11,13);
                Integer time = Integer.valueOf(substring);
                int start = i*2;
                int end = (i*2) + 2;
                if(time >= start && time < end){
                    number += pco.getNumber();
                }
            }
            numberList.add(number);
        }
        DataShowMiddleSiteVO dataShowMiddleSiteVO = new DataShowMiddleSiteVO();
        dataShowMiddleSiteVO.setId(id);
        dataShowMiddleSiteVO.setNumberList(numberList);
        return BaseResponse.ok(dataShowMiddleSiteVO);
    }

    @Override
    public List<SiteVO> selectSiteList() {
        JPersonVO jPersonVO = this.rootTrue();
        LambdaQueryWrapper<Company> companyWrapper = new LambdaQueryWrapper<>();
        companyWrapper.eq(StringUtils.isNotEmpty(jPersonVO.getCompanyId()), Company::getCompanyId, jPersonVO.getCompanyId());
        companyWrapper.eq(StringUtils.isNotEmpty(jPersonVO.getBranchId()), Company::getBranchId, jPersonVO.getBranchId());
        List<Company> companyList = companysMapper.selectList(companyWrapper);
        return companyList.stream().map(p ->
                new SiteVO().setId(p.getId()).setName(p.getCompanyName() + "-" + (StringUtils.isEmpty(p.getBranchName())?"":p.getBranchName()))
        ).collect(Collectors.toList());
    }

    @Override
    public List<DataShowWarningVO> selectWarningList() {
        JPersonVO jPersonVO = this.rootTrue();
        /*根据账号查询companyId和branchId 然后预警人员类型为0 1 3*/
        LambdaQueryWrapper<PassreCords> PassreCordsWrapper = new LambdaQueryWrapper<>();
        PassreCordsWrapper.eq(StringUtils.isNotEmpty(jPersonVO.getCompanyId()), PassreCords::getCompanyId, jPersonVO.getCompanyId());
        PassreCordsWrapper.eq(StringUtils.isNotEmpty(jPersonVO.getBranchId()), PassreCords::getBranchId, jPersonVO.getBranchId());
        List<Integer> list = new ArrayList<>();
        Collections.addAll(list,0,1);//去掉了健康码异常的实时预警
        PassreCordsWrapper.in(PassreCords::getPass,list);
        PassreCordsWrapper.eq(PassreCords::getFirstPass,true);
        PassreCordsWrapper.orderByDesc(PassreCords::getPassTime);
        Page<PassreCords> selectPage = passreCordsMapper.selectPage(new Page<>(0, 6), PassreCordsWrapper);
        List<PassreCords> records = selectPage.getRecords();

        return records.stream().map(p -> {
            DataShowWarningVO dataShowWarningVO = new DataShowWarningVO();
            BeanUtils.copyProperties(p,dataShowWarningVO);
            return dataShowWarningVO;
        }).collect(Collectors.toList());
    }

    @Override
    public WarningDetailOneVO selectWarningDetail(Integer id) {
        return passreCordsService.warningDeatil(id);
    }

    private JPersonVO rootTrue(){
        JPersonVO jPersonVO = jPersonsService.getUserInfo();
        Integer adminType = jPersonVO.getType();
        if (adminType != 1 && adminType != 2) {
            throw new ServiceException("没有权限");
        }
        return jPersonVO;
    }
}
