package com.metro.service.impl;


import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.metro.entity.dto.JDeviceUpdateDTO;
import com.metro.entity.po.Company;
import com.metro.entity.po.JDevices;
import com.metro.entity.po.JDevicesMetro;
import com.metro.entity.po.Metro;
import com.metro.entity.search.JDevicesSearch;
import com.metro.entity.vo.JDevicesLessVO;
import com.metro.entity.vo.JDevicesVO;
import com.metro.entity.vo.MetroVO;
import com.metro.exception.BusinessException;
import com.metro.mapper.JDevicesMapper;
import com.metro.mapper.JDevicesMetroMapper;
import com.metro.mapper.MetroMapper;
import com.metro.service.CompanyService;
import com.metro.service.JDevicesService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-04-26
 */
@Service
public class JDevicesServiceImpl implements JDevicesService {

    @Resource
    private JDevicesMapper jDevicesMapper;
    @Resource
    private CompanyService companyService;
    @Resource
    private MetroMapper metroMapper;
    @Resource
    private JDevicesMetroMapper jDevicesMetroMapper;

    @Override
    public IPage<JDevicesVO> getPageList(JDevicesSearch jDevicesSearch) {
        Page<JDevicesVO> page = new Page<>(jDevicesSearch.getPageNum(),jDevicesSearch.getPageSize());
        IPage<JDevicesVO>  pageList =jDevicesMapper.getPageList(page,jDevicesSearch);
        List<JDevicesVO> records = pageList.getRecords();

        /*获取数据中companyMap 和barnchMap*/
        Map<String, Company> companyMap = companyService.getCompanyMapByList(records, JDevicesVO::getCompanyId);
        Map<String, Company> barnchMap = companyService.getBranchMapByList(records,JDevicesVO::getBranchId);
        Set<Integer> metroIdSet = records.stream().map(JDevicesVO::getMetroId).collect(Collectors.toSet());
        List<MetroVO> metroIdList = null ;
        if(metroIdSet.size() == 0){metroIdList = new ArrayList<>();}
        else { metroIdList = metroMapper.selectByMetroIdSet(metroIdSet);}
        Map<Integer, MetroVO> metroMap = returnMap(metroIdList);

        /*数据填入*/
        for (JDevicesVO record : records) {
            record.setCompanyName(companyMap.get(record.getCompanyId()) == null?"" : companyMap.get(record.getCompanyId()).getCompanyName());
            record.setBranchName(barnchMap.get(record.getCompanyId()+record.getBranchId()) == null ?
                    "" : barnchMap.get(record.getCompanyId()+record.getBranchId()).getBranchName()).getBranchName();
            if(metroMap.get(record.getMetroId()) != null){
                MetroVO metroVO = metroMap.get(record.getMetroId());
                record.setMetroId(metroVO.getMetroId());
                record.setMetroName(metroVO.getMetroName());
                record.setSiteId(metroVO.getSiteId());
                record.setSiteName(metroVO.getSiteName());
                record.setLineId(metroVO.getLineId());
                record.setLineName(metroVO.getLineName());
            }

        }
        return pageList;
    }

    @Override
    public Boolean updateJDevice(JDeviceUpdateDTO jDeviceUpdateDTO) {
        Integer metroId = jDeviceUpdateDTO.getMetroId();
        Metro metro = metroMapper.selectById(metroId);
        if(metro.getLevel() != 3)throw new BusinessException("请选择具体的地铁盘口");
        JDevicesMetro jDevicesMetro = jDevicesMetroMapper.selectOne(new LambdaQueryWrapper<JDevicesMetro>().eq(JDevicesMetro::getDevicesId, jDeviceUpdateDTO.getId()));
        Integer i = null;
        if(jDevicesMetro == null){
            jDevicesMetro = new JDevicesMetro();
            jDevicesMetro.setDevicesId(jDeviceUpdateDTO.getId());
            jDevicesMetro.setMetroId(jDeviceUpdateDTO.getMetroId());
            i = jDevicesMetroMapper.insert(jDevicesMetro);
        }else {
            jDevicesMetro.setMetroId(jDeviceUpdateDTO.getMetroId());
            i = jDevicesMetroMapper.updateById(jDevicesMetro);
        }

        if(i == 1)return true;
        else return false;
    }

    @Override
    public List<JDevicesLessVO> getList() {
        /*获取id和设备编号*/
        List<JDevices> jDevices = jDevicesMapper.selectList(new LambdaQueryWrapper<JDevices>().select(JDevices::getId, JDevices::getDeviceId));
        return jDevices.stream().map(p -> BeanUtil.toBean(p, JDevicesLessVO.class)).collect(Collectors.toList());
    }

    private Map<Integer,MetroVO> returnMap(List<MetroVO> list){
        Map<Integer,MetroVO> map = new HashMap<>();
        list.forEach(p ->
            map.put(p.getMetroId(),p)
        );
        return map;
    }
}
