package com.metro.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.metro.entity.dto.JpersonDTO;
import com.metro.entity.dto.JpersonEditDTO;
import com.metro.entity.po.Company;
import com.metro.entity.po.JPersons;
import com.metro.entity.search.JPersonSearch;
import com.metro.entity.vo.JPersonVO;
import com.metro.exception.BusinessException;
import com.metro.mapper.CompanysMapper;
import com.metro.mapper.JPersonsMapper;
import com.metro.restful.RestCode;
import com.metro.security.JWTUtils;
import com.metro.service.CompanyService;
import com.metro.service.JPersonsService;
import com.metro.utils.MapperManager;
import com.metro.utils.SMSOrIdCardUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wxg
 * @since 2022-04-29
 */
@Service
@Slf4j
public class JPersonsServiceImpl implements JPersonsService {

    @Resource
    private JPersonsMapper jPersonsMapper;
    @Resource
    private CompanysMapper companysMapper;
    @Autowired
    private CompanyService companyService;

    @Override
    public JPersonVO login(JpersonDTO jpersonDTO) {
        LambdaQueryWrapper<JPersons> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(JPersons::getCardId,jpersonDTO.getCardId());
        JPersons jPersons = jPersonsMapper.selectOne(wrapper);
        if (ObjectUtil.isEmpty(jPersons)) throw new BusinessException("账号不存在!请先注册");
        if(StringUtils.isEmpty(jPersons.getPasswd())){
            throw new BusinessException("该账号无法登录");
        }

        if (!(jPersons.getPasswd()).equals(jpersonDTO.getPassword()))
            throw new BusinessException("密码错误!请重新输入");
        Integer type  = StringUtils.isEmpty(jPersons.getCompanyId()) ? 1 : (StringUtils.isEmpty(jPersons.getBranchId())? 2 :3);

        JPersonVO mapper = MapperManager.mapper(jPersons, JPersonVO.class, p -> {
            p.setType(type);
            p.setToken(JWTUtils.sign(jPersons.getId().toString(), jPersons.getPasswd(), DateUtils.addDays(new Date(), 1)));
            /*数据填充，将companyName 和 branchName填入*/
            if(StringUtils.isNotEmpty(jPersons.getCompanyId())){
                p.setCompanyName(companysMapper.getCompanyByCompanyId(jPersons.getCompanyId()).getCompanyName());
                if(StringUtils.isNotEmpty(jPersons.getBranchId())){
                    p.setBranchName(companysMapper.getCompanyByCompanyIdandBranchId(jPersons.getCompanyId(),jPersons.getBranchId()).getBranchName());
                }
            }
        });
        /*将jperson的数据复制到JpesonVO中 并且设置token返回*/
        return mapper;
    }

    /**
     * 退出登录
     */
    @Override
    public Boolean logout() {
        return null;
    }

    @Override
    public JPersonVO getUserInfo() {
        Long userId = this.getUserId();
        JPersons jPersons = jPersonsMapper.selectById(userId);
        if(jPersons == null){throw new BusinessException("数据有误");}
        String companyId = jPersons.getCompanyId() == null ? "" : jPersons.getCompanyId();
        String branchId = jPersons.getBranchId() == null ? "" : jPersons.getBranchId();
        Integer type  = StringUtils.isEmpty(companyId) ? 1 : (StringUtils.isEmpty(branchId)? 2 :3);
        return MapperManager.mapper(jPersons, JPersonVO.class, p ->{
            p.setType(type);
            if(StringUtils.isNotEmpty(jPersons.getCompanyId())){
                p.setCompanyName(companysMapper.getCompanyByCompanyId(jPersons.getCompanyId()).getCompanyName());
                if(StringUtils.isNotEmpty(jPersons.getBranchId())){
                    p.setBranchName(companysMapper.getCompanyByCompanyIdandBranchId(jPersons.getCompanyId(),jPersons.getBranchId()).getBranchName());
                }
            }
        });
    }

    @Override
    public IPage<JPersonVO> getPageList(JPersonSearch jPersonSearch) {
        Long userId = this.getUserId();
        JPersons jPersons = jPersonsMapper.selectById(userId);
        jPersonSearch.setCompanyId(jPersons.getCompanyId() !=null ? jPersons.getCompanyId() : null);
        jPersonSearch.setBranchId(jPersons.getBranchId() !=null ? jPersons.getBranchId() : null);
        Integer type = StringUtils.isEmpty(jPersons.getCompanyId()) ? 1 : (StringUtils.isEmpty(jPersons.getBranchId())? 2 :3);
        jPersonSearch.setType(type);
        jPersonSearch.setAdminId(userId.intValue());
        if(type == 3)throw new BusinessException("您无权查看账号列表");
        Page<JPersonVO> page = new Page<>(jPersonSearch.getPageNum(),jPersonSearch.getPageSize());
        IPage<JPersonVO>  pageList =jPersonsMapper.getPageList(page,jPersonSearch);

        List<JPersonVO> records = pageList.getRecords();
        Map<String, Company> companyMap = companyService.getCompanyMapByList(records, JPersonVO::getCompanyId);
        Map<String, Company> barnchMap = companyService.getBranchMapByList(records,JPersonVO::getBranchId);
        /*数据填入*/
        for (JPersonVO record : records) {
            record.setCompanyName(companyMap.get(record.getCompanyId()) == null?"" : companyMap.get(record.getCompanyId()).getCompanyName());
            record.setBranchName(barnchMap.get(record.getCompanyId()+record.getBranchId()) == null ?
                    "" : barnchMap.get(record.getCompanyId()+record.getBranchId()).getBranchName()).getBranchName();
            record.setType(StringUtils.isEmpty(record.getCompanyId()) ? 1 : (StringUtils.isEmpty(record.getBranchId())? 2 :3));
        }


        return pageList;
    }

    @Override
    public Integer insert(JpersonEditDTO jpersonEditDTO) {
        Date date  = new Date();
        if(StringUtils.isNotEmpty(jpersonEditDTO.getCompanyId()) && StringUtils.isNotEmpty(jpersonEditDTO.getBranchId())){
//            if(StringUtils.isEmpty(jpersonEditDTO.getPhoto()))throw new BusinessException("部门和区域都有值的情况下，图片不能为空");
        }else if(StringUtils.isEmpty(jpersonEditDTO.getCompanyId()) && StringUtils.isEmpty(jpersonEditDTO.getBranchId())){
            throw new BusinessException("创建一级账号，请联系管理员");
        }

        List<JPersons> jPersons1 = jPersonsMapper.selectList(new LambdaQueryWrapper<JPersons>()
                .eq(JPersons::getCardId, jpersonEditDTO.getCardId())
                );
        if(jPersons1 != null && jPersons1.size() > 0){
            if(jpersonEditDTO.getId() == null){
                throw new BusinessException("该账号已经被创建");
            }else {
                if(!jpersonEditDTO.getId() .equals( jPersons1.get(0).getId() )){
                    throw new BusinessException("该账号已经被创建");
                }
            }
        }

        JPersons jPersons = new JPersons();
        BeanUtils.copyProperties(jpersonEditDTO,jPersons);
        JPersonVO userInfo = this.getUserInfo();
        if(StringUtils.isNotEmpty(userInfo.getCompanyId())){
            jPersons.setCompanyId(userInfo.getCompanyId());
            if(StringUtils.isEmpty(jpersonEditDTO.getBranchId())){
                throw new BusinessException("请选择站台");
            }
        }else{
            if(StringUtils.isEmpty(jpersonEditDTO.getCompanyId())){
                throw new BusinessException("请选择地铁");
            }
        }
        if(StringUtils.isNotEmpty(jPersons.getCompanyId()) && StringUtils.isNotEmpty(jPersons.getBranchId())){
            if(StringUtils.isEmpty(jPersons.getIdNum())){
                throw new BusinessException("民警身份证不能为空");
            }else{
                if(!SMSOrIdCardUtils.isIdCard(jPersons.getIdNum())){
                    throw new BusinessException("身份证有误");
                }
            }
        }
        Integer integer = null ;
        if(jpersonEditDTO.getId() == null ){
            jPersons.setRegTime(date);
            jPersons.setIsDelete(0);
            jPersons.setIsAdmin(0);
            jPersons.setValidStartTime(date.getTime());
            jPersons.setValidEndTime(date.getTime());
            if(StringUtils.isEmpty(jpersonEditDTO.getPasswd()))throw new BusinessException("密码不能为空");
            integer = jPersonsMapper.insert(jPersons);
        }else {
            jPersons.setUpdateTimestamp(date.getTime());
            integer = jPersonsMapper.updateById(jPersons);
        }
        return integer;
    }

    @Override
    public Integer delete(Integer id) {
        return jPersonsMapper.deleteById(id);
    }

    public Long getUserId() {
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (requestAttributes == null) {
            throw new BusinessException(RestCode.UNAUTHORIZED.getMsg());
        }
        HttpServletRequest request = requestAttributes.getRequest();
        return getUserId(request);
    }

    public Long getUserId(HttpServletRequest request) {
        String token = request.getHeader("Authorization");
        if(StringUtils.isEmpty(token)){
            throw new BusinessException(RestCode.UNAUTHORIZED.getMsg());
        }
        token = JWTUtils.getUserId(token);
        if (StringUtils.isEmpty(token)) {
            throw new BusinessException(RestCode.UNAUTHORIZED.getMsg());
        }
        return Long.valueOf(token);
    }
}
