package com.metro.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.metro.entity.po.Metro;
import com.metro.entity.ro.metro.SelectMetroParentResp;
import com.metro.entity.ro.metro.SelectMetroReq;
import com.metro.entity.ro.metro.SelectMetroResp;
import com.metro.mapper.MetroMapper;
import com.metro.service.MetroService;
import com.metro.utils.MapperManager;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName: MetroServiceImpl
 * @Description: TODO
 * @Author: ZS
 * @CreateName: lws
 * @Date 2022/3/9 1:39
 * ...
 */
@Service
public class MetroServiceImpl implements MetroService {

    @Autowired
    private MetroMapper metroMapper;

    /**
     * 地铁查询
     *
     * @param req 查询条件
     * @return 返回结果
     */
    @Override
    public List<SelectMetroResp> list(SelectMetroReq req) {
        return  metroMapper.selectList(selectListWrapper(req)).stream().map(p -> MapperManager.mapper(p,SelectMetroResp.class)).collect(Collectors.toList());
    }

    // 地铁查询wrapper
    private Wrapper<Metro> selectListWrapper(SelectMetroReq req) {
        LambdaQueryWrapper<Metro> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(ObjectUtil.isNull(req.getPid()),Metro::getPid,0);
        wrapper.eq(ObjectUtil.isNotNull(req.getPid()),Metro::getPid,req.getPid());
        return wrapper;
    }

    @Override
    public List<SelectMetroParentResp> selectMetroAll() {
        //1、查询数据库钟，所有的级联信息（ 我这里是使用上面的默认数据在测试）
        List<Metro> list = metroMapper.selectList(null);
        if(CollectionUtils.isEmpty(list)) return Lists.newArrayList();
        List<SelectMetroParentResp> parentResps = list.stream().map(p -> MapperManager.mapper(p, SelectMetroParentResp.class)).collect(Collectors.toList());
        //2、先找到一级父类，再通过级联去查询子类菜单
        return  parentResps.stream().filter(linkageEntities ->
                linkageEntities.getPid().equals("0")     //查询父类
        ).map((menu) -> {
            menu.setChildren(getChildrens(menu, parentResps)); //查询子类菜单
            return menu;
        }).sorted((menu1, menu2) -> {
            //排序
            return (menu1.getSort() == null ? 0 : menu1.getSort()) - (menu2.getSort() == null ? 0 : menu2.getSort());
        }).collect(Collectors.toList());
    }



    //递归查找所有菜单的子类菜单
    private static List<SelectMetroParentResp> getChildrens(SelectMetroParentResp root, List<SelectMetroParentResp> all) {
        return all.stream().filter(linkageEntities -> {
            return linkageEntities.getPid().equals(root.getId());
        }).map(linkageEntities -> {
            //1、递归找到子菜单
            linkageEntities.setChildren(getChildrens(linkageEntities, all));
            return linkageEntities;
        }).sorted((menu1, menu2) -> {
            //2、菜单的排序
            return (menu1.getSort() == null ? 0 : menu1.getSort()) - (menu2.getSort() == null ? 0 : menu2.getSort());
        }).collect(Collectors.toList());
    }

}
