package com.metro.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.metro.entity.excel.WarningExcel;
import com.metro.entity.po.Company;
import com.metro.entity.po.JDevices;
import com.metro.entity.po.JDevicesMetro;
import com.metro.entity.po.PassreCords;
import com.metro.entity.ro.passrecords.*;
import com.metro.entity.search.WarningSearch;
import com.metro.entity.vo.JPersonVO;
import com.metro.entity.vo.MetroVO;
import com.metro.entity.vo.WarningDetailOneVO;
import com.metro.entity.vo.WarningVO;
import com.metro.exception.BusinessException;
import com.metro.mapper.JDevicesMapper;
import com.metro.mapper.JDevicesMetroMapper;
import com.metro.mapper.MetroMapper;
import com.metro.mapper.PassreCordsMapper;
import com.metro.service.CompanyService;
import com.metro.service.JPersonsService;
import com.metro.service.PassreCordsService;
import com.metro.service.base.BaseService;
import com.metro.utils.MapperManager;
import com.metro.utils.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.net.URLEncoder;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @ClassName: PassreCordsServiceImpl
 * @Description: TODO
 * @Author: ZS
 * @CreateName: lws
 * @Date 2022/3/8 12:14
 * ...
 */
@Service
public class PassreCordsServiceImpl extends BaseService implements PassreCordsService {

    @Resource
    private PassreCordsMapper passreCordsMapper;
    @Resource
    private CompanyService companyService;
    @Autowired
    private JPersonsService jPersonsService;
    @Resource
    private JDevicesMapper jDevicesMapper;
    @Resource
    private JDevicesMetroMapper jDevicesMetroMapper;
    @Resource
    private MetroMapper metroMapper;
    /**
     * 警示记录
     *
     * @param req 查询条件
     * @return page列表
     */
    @Override
    public IPage<SelectPassreCordsPageListResp> page(SelectPassreCordsPageListReq req) {
        JPersonVO userInfo = jPersonsService.getUserInfo();
        if(userInfo.getType() == 3){
            req.setStaffCardId(userInfo.getCardId());
        } else if (userInfo.getType() == 2) {
            req.setCompanyId(userInfo.getCompanyId());
        }
        /*分页查询sql*/
        IPage<SelectPassreCordsPageListResp> respPage = new Page<>(req.getPageNum(), req.getPageSize());
        Page<PassreCords> selectPage = passreCordsMapper.selectPage(new Page<>(req.getPageNum(), req.getPageSize()), pageWrapper(req,1));
        if (CollectionUtils.isEmpty(selectPage.getRecords())) return respPage;

        List<PassreCords> records = selectPage.getRecords();
        /*获取数据中companyMap 和barnchMap*/
        Map<String, Company> companyMap = companyService.getCompanyMapByList(records,PassreCords::getCompanyId);
        Map<String, Company> barnchMap = companyService.getBranchMapByList(records,PassreCords::getBranchId);

        /*数据回填*/
        return selectPage.convert(p -> MapperManager.mapper(p, SelectPassreCordsPageListResp.class, q ->{
            q.setCompanyName(companyMap.get(q.getCompanyId()) == null?"" : companyMap.get(q.getCompanyId()).getCompanyName());
            q.setBranchName(barnchMap.get(q.getCompanyId()+q.getBranchId()) == null ?
                    "" : barnchMap.get(q.getCompanyId()+q.getBranchId()).getBranchName()).getBranchName();
            }
        ));
    }

    /*
     * 删除
     * */
    @Override
    public DeletePassreCordsResq delete(DeletePassreCordsReq req) {
        return new DeletePassreCordsResq().setCount(passreCordsMapper.deleteById(req.getId()));

    }

    /*
     * 详情
     * */
    @Override
    public DetialPassreCordsResq selectById(DetailPassreCordsReq req) {
        PassreCords passreCords = passreCordsMapper.selectById(req.getId());
        return MapperManager.mapper(passreCords, DetialPassreCordsResq.class);

    }

    @Override
    public IPage<WarningVO> warningPageList(WarningSearch warningSearch) {
        JPersonVO userInfo = jPersonsService.getUserInfo();
        if(userInfo.getType() == 3){
            warningSearch.setStaffCardId(userInfo.getCardId());
        } else if (userInfo.getType() == 2) {
            warningSearch.setCompanyId(userInfo.getCompanyId());
        }
        Page<WarningVO> page = new Page<>(warningSearch.getPageNum(),warningSearch.getPageSize());
        IPage<WarningVO>  pageList =passreCordsMapper.warningPageList(page,warningSearch);
        List<WarningVO> records = pageList.getRecords();

        /*获取id列表 并根据idList获取数据列表*/
        List<Integer> idList = records.stream().map(WarningVO::getId).collect(Collectors.toList());
        if(idList.size() == 0)return page;
        LambdaQueryWrapper<PassreCords> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.in(PassreCords::getId, idList);
        lambdaQueryWrapper.eq(PassreCords::getFirstPass, true);
        lambdaQueryWrapper.orderByDesc(PassreCords::getId);
        List<PassreCords> passreCords = passreCordsMapper.selectList(lambdaQueryWrapper);
        Map<Integer, PassreCords> map = passreCords.stream().collect(Collectors.toMap(PassreCords::getId, Function.identity()));
        List<Integer> list = new ArrayList<>();
        Collections.addAll(list,0,1);//去掉了健康码异常的实时预警
        for (WarningVO record : records) {
            PassreCords passreCords1 = map.get(record.getId());
            BeanUtils.copyProperties(passreCords1,record);
            LambdaQueryWrapper<PassreCords> wrapper = new LambdaQueryWrapper<>();
            if(StringUtils.isEmpty(passreCords1.getCardId()))continue;
            wrapper.eq(StringUtils.isNotEmpty(warningSearch.getCompanyId()),PassreCords::getCompanyId,warningSearch.getCompanyId());
            wrapper.eq(StringUtils.isNotEmpty(warningSearch.getBranchId()),PassreCords::getBranchId,warningSearch.getBranchId());
            wrapper.eq(PassreCords::getCardId,passreCords1.getCardId());
            wrapper.ne(PassreCords::getId,passreCords1.getId());
            wrapper.in(PassreCords::getPass,list);
            wrapper.eq(PassreCords::getFirstPass, true);
            wrapper.orderByDesc(PassreCords::getId);
            wrapper.last("limit 1");
            PassreCords passreCord = passreCordsMapper.selectOne(wrapper);
            if(passreCord != null)
            record.setLastTime(passreCord.getPassTime()!=null?passreCord.getPassTime():null);
        }

        return pageList;
    }

    @Override
    public List<SelectPassreCordsPageListResp> warningDeatilList(SelectPassreCordsPageListReq req) {
        List<PassreCords> records = passreCordsMapper.selectList(pageWrapper(req,2));

        Map<String,String> map = new HashMap<>();
        List<String> collect = records.stream().map(p -> p.getMsgId().substring(0, 8)).collect(Collectors.toSet()).stream().collect(Collectors.toList());
        if(collect.size() == 0)return new ArrayList<>();
        List<Map<String, String>> maps = metroMapper.selectMetroNameByDeviceId(collect);
        if(maps != null) {
            for (Map<String, String> stringStringMap : maps) {
                map.put(stringStringMap.get("deviceId"), stringStringMap.get("metroName"));
            }
        }

        /*获取数据中companyMap 和barnchMap*/
        Map<String, Company> companyMap = companyService.getCompanyMapByList(records,PassreCords::getCompanyId);
        Map<String, Company> barnchMap = companyService.getBranchMapByList(records,PassreCords::getBranchId);

        /*数据填入*/
        for (PassreCords record : records) {
            record.setCompanyName(companyMap.get(record.getCompanyId()) == null?"" : companyMap.get(record.getCompanyId()).getCompanyName());
            record.setBranchName(barnchMap.get(record.getCompanyId()+record.getBranchId()) == null ?
                    "" : barnchMap.get(record.getCompanyId()+record.getBranchId()).getBranchName()).getBranchName();
        }

        List<SelectPassreCordsPageListResp> list = records.stream().map(passreCords -> {
            SelectPassreCordsPageListResp userCheckData = new SelectPassreCordsPageListResp();
            BeanUtils.copyProperties(passreCords, userCheckData);
            String deviceId = StringUtils.isEmpty(passreCords.getMsgId()) ? null : passreCords.getMsgId().substring(0,8);
            if(deviceId != null){
                userCheckData.setMetroName(map.get(deviceId) != null?map.get(deviceId):"");
            }
            return userCheckData;
        }).collect(Collectors.toList());

        return list;
    }

    @Override
    public WarningDetailOneVO warningDeatil(Integer id) {
        PassreCords passreCords = passreCordsMapper.selectById(id);
        if(passreCords == null) throw new BusinessException("没有该条记录");
        WarningDetailOneVO warningDetailOneVO = new WarningDetailOneVO();
        BeanUtils.copyProperties(passreCords,warningDetailOneVO);
        /*获取设备信息 （设备编号是msgId字段的前8位）*/
        String deviceId = passreCords.getMsgId().substring(0,8);
        JDevices jDevices = jDevicesMapper.selectOne(new LambdaQueryWrapper<JDevices>().eq(JDevices::getDeviceId, deviceId));
        /*获取盘口信息并填入*/
        if(jDevices != null){
            warningDetailOneVO.setDeviceId(jDevices.getDeviceId());
            JDevicesMetro jDevicesMetro = jDevicesMetroMapper.selectOne(new LambdaQueryWrapper<JDevicesMetro>().eq(JDevicesMetro::getDevicesId, jDevices.getId()));
            if(jDevicesMetro != null){
                MetroVO metroVO = metroMapper.selectByMetroId(jDevicesMetro.getMetroId());
                if(metroVO != null)
                {warningDetailOneVO.setMetroName(metroVO.getLineName()+"-"+metroVO.getSiteName()+"-"+metroVO.getMetroName());}
            }
        }
        return warningDetailOneVO;
    }

    @Override
    public void excelWarningPageList(WarningSearch warningSearch, HttpServletResponse response) {
        warningSearch.setPageSize(50000);
        warningSearch.setPageNum(0);
        JPersonVO userInfo = jPersonsService.getUserInfo();
        if(userInfo.getType() == 3){
            warningSearch.setBranchId(userInfo.getBranchId());
            warningSearch.setCompanyId(userInfo.getCompanyId());
        } else if (userInfo.getType() == 2) {
            warningSearch.setCompanyId(userInfo.getCompanyId());
        }
        Page<WarningVO> page = new Page<>(warningSearch.getPageNum(),warningSearch.getPageSize());
        IPage<WarningVO>  pageList =passreCordsMapper.warningPageList(page,warningSearch);
        List<WarningVO> records = pageList.getRecords();

        /*获取id列表 并根据idList获取数据列表*/
        List<Integer> idList = records.stream().map(WarningVO::getId).collect(Collectors.toList());
        List<WarningExcel> list = new ArrayList<>();
        if(idList.size() != 0){
            List<PassreCords> passreCords = passreCordsMapper.selectByIdList(idList);
            Map<Integer, PassreCords> map = passreCords.stream().collect(Collectors.toMap(PassreCords::getId, Function.identity()));
            List<Integer> passlist = new ArrayList<>();
            Collections.addAll(passlist,0,1);//去掉了健康码异常的实时预警
            for (WarningVO record : records) {
                WarningExcel warningExcel = new WarningExcel();
                PassreCords passreCords1 = map.get(record.getId());
                BeanUtils.copyProperties(passreCords1,warningExcel);
                LambdaQueryWrapper<PassreCords> wrapper = new LambdaQueryWrapper<>();
                if(StringUtils.isEmpty(passreCords1.getCardId()))continue;
                wrapper.eq(StringUtils.isNotEmpty(warningSearch.getCompanyId()),PassreCords::getCompanyId,warningSearch.getCompanyId());
                wrapper.eq(StringUtils.isNotEmpty(warningSearch.getBranchId()),PassreCords::getBranchId,warningSearch.getBranchId());
                wrapper.eq(PassreCords::getCardId,passreCords1.getCardId());
                wrapper.ne(PassreCords::getId,passreCords1.getId());
                wrapper.in(PassreCords::getPass,passlist);
                wrapper.orderByDesc(PassreCords::getId);
                wrapper.last("limit 1");
                PassreCords passreCord = passreCordsMapper.selectOne(wrapper);
                if(passreCord != null){
                    warningExcel.setLastTime(passreCord.getPassTime()!=null?passreCord.getPassTime():null);
                }
                list.add(warningExcel);
            }
        }
        try {
            String fileName = "预警人员-" + System.currentTimeMillis() + ".xlsx";
            response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            response.addHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setContentType("application/vnd.ms-excel;charset=UTF-8");
//            response.setContentType("application/vnd.ms-excel");
//            response.setCharacterEncoding("utf-8");
//            // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
//            String fileName = "预警人员";
//            try {
//                fileName = URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");
//            } catch (UnsupportedEncodingException e) {
//                e.printStackTrace();
//            }
//            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");

            EasyExcel.write(response.getOutputStream(), WarningExcel.class)
                    // 导出文件名
                    .autoCloseStream(Boolean.TRUE).sheet("预警人员")
                    .doWrite(list);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    // page Wrapper
    private LambdaQueryWrapper<PassreCords> pageWrapper(SelectPassreCordsPageListReq req , Integer type) {
        LambdaQueryWrapper<PassreCords> wrapper = new LambdaQueryWrapper<>();
//        if (getAccountInfo().getType() == 2) {
//           wrapper.eq(PassreCords::getCompanyId, getAccountInfo().getCompanyId());
//        }
//        if (getAccountInfo().getType() == 3) {
//            //显示该分局账号下的所有盘查记录 companyID = 记录companyID branchID = 记录branchId
//            wrapper.eq(PassreCords::getCompanyId, getAccountInfo().getCompanyId()).eq(PassreCords::getBranchId, getAccountInfo().getBranchId());
//        }
//        if (getAccountInfo().getType() == 4) {
//            //显示该民警产生的盘查记录 companyID = 记录companyID branchID = 记录branchId userName = peopleName
//            wrapper.eq(PassreCords::getCompanyId, getAccountInfo().getCompanyId()).
//                    eq(PassreCords::getBranchId, getAccountInfo().getBranchId()).
//                    eq(PassreCords::getStaffName,getAccountInfo().getUserName());
//        }
        if(type == 2){
            wrapper.select(PassreCords.class,p ->!p.getColumn().equals("scene_photo"));
//            List<Integer> list = new ArrayList<>();
//            list.add(0);
//            list.add(1);
//            wrapper.in(PassreCords::getPass, list);
        }

        JPersonVO userInfo = jPersonsService.getUserInfo();
        if(userInfo.getType() != 3){
            if(StringUtils.isNotEmpty(userInfo.getCompanyId())){
                wrapper.eq(PassreCords::getCompanyId, userInfo.getCompanyId());
                if(StringUtils.isNotEmpty(userInfo.getBranchId())){
                    wrapper.eq(PassreCords::getBranchId, userInfo.getBranchId());
                }
            }
        }

        wrapper.like(StringUtils.isNotEmpty(req.getXm()), PassreCords::getXm, req.getXm());
        wrapper.eq(StringUtils.isNotEmpty(req.getCompanyId()), PassreCords::getCompanyId, req.getCompanyId());
        wrapper.eq(StringUtils.isNotEmpty(req.getBranchId()), PassreCords::getBranchId, req.getBranchId());
        wrapper.eq(StringUtils.isNotEmpty(req.getCardId()), PassreCords::getCardId, req.getCardId());
        wrapper.eq(StringUtils.isNotEmpty(req.getStaffCardId()), PassreCords::getStaffCardId, req.getStaffCardId());
        wrapper.eq(StringUtils.isNotEmpty(req.getPhone()), PassreCords::getPhone, req.getPhone());
        wrapper.eq(req.getPass() != null, PassreCords::getPass, req.getPass());
        wrapper.eq(PassreCords::getFirstPass, true);
        wrapper.gt(StringUtils.isNotEmpty(req.getStartTime()), PassreCords::getPassTime, req.getStartTime());
        wrapper.lt(StringUtils.isNotEmpty(req.getEndTime()), PassreCords::getPassTime, req.getEndTime());
        wrapper.orderByDesc(PassreCords::getPassTime);
        if(type == 2)wrapper.last("limit 50");
        return wrapper;
    }
}
