package com.metro.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.metro.constant.UserConstant;
import com.metro.entity.po.*;
import com.metro.entity.ro.Police.*;
import com.metro.entity.ro.account.*;
import com.metro.entity.ro.passrecords.SelectPassreCordsPageListResp;
import com.metro.exception.BusinessException;
import com.metro.mapper.AccountsMapper;
import com.metro.mapper.CompanysMapper;
import com.metro.mapper.PoliceMapper;
import com.metro.service.PoliceService;
import com.metro.service.base.BaseService;
import com.metro.utils.MD5UtilNew;
import com.metro.utils.MapperManager;
import com.metro.utils.StringUtil;
import io.github.classgraph.utils.Join;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.metro.service.impl.AccountServiceImpl.getUpdateAccountResp;


/**
 * @ClassName: PoliceServiceImpl
 * @Description: TODO
 * @Author: ZS
 * @CreateName: lws
 * @Date 2022/3/18 1:25
 * ...
 */
@Service
public class PoliceServiceImpl extends BaseService implements PoliceService {
    @Autowired
    private PoliceMapper mapper;
    @Autowired
    private CompanysMapper companysMapper;
    @Autowired
    private AccountsMapper accountsMapper;
    /*
    * 派出所列表展示
    * */
    @Override
    public IPage<SelectPolicePageResp> page(SelectPolicePageReq req) {
        IPage<SelectPolicePageResp> respPage = new Page<>(req.getPageNum(), req.getPageSize());
        Page<Police> selectPage = mapper.selectPage(new Page<>(req.getPageNum(), req.getPageSize()), pageWrapper(req));
        if (CollectionUtils.isEmpty(selectPage.getRecords())) return respPage;
        Set<String> getCompanyId = selectPage.getRecords().stream().map(Police::getCompanyId).collect(Collectors.toSet());
        return selectPage.convert(p -> MapperManager.mapper(p, SelectPolicePageResp.class));

    }


    /*
     * 查询条件
     * */
    private LambdaQueryWrapper<Police> pageWrapper(SelectPolicePageReq req) {
        LambdaQueryWrapper<Police> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(Police::getIsDelete, UserConstant.NOT_DELETE);
        wrapper.like(com.metro.utils.StringUtils.isNotEmpty(req.getAddress()), Police::getAddress, req.getAddress());
        wrapper.like(com.metro.utils.StringUtils.isNotEmpty(req.getUserName()), Police::getUserName, req.getUserName());
        wrapper.like(com.metro.utils.StringUtils.isNotEmpty(req.getPhone()), Police::getPhone, req.getPhone());
        wrapper.like(com.metro.utils.StringUtils.isNotEmpty(req.getAccount()),Police::getAccount,req.getAccount());
        return wrapper;
    }

    /*
     * 新增
     * */
    @Override
    @Transactional
    public InsertPoliceResp add(InsertPoliceReq req) {
        if (getAccountInfo().getType() != 2) throw new BusinessException("非分局管理员不能注册");
        Police account = mapper.selectOne(new LambdaQueryWrapper<Police>().eq(Police::getAccount, req.getAccount()).eq(Police::getIsDelete, UserConstant.NOT_DELETE));
        if (ObjectUtil.isNotEmpty(account)) throw new BusinessException(account.getAccount() + "已经存在！不能注册");
        List<Company> companies = companysMapper.selectList(new LambdaQueryWrapper<Company>().eq(Company::getCompanyId, getAccountInfo().getCompanyId()).eq(Company::getBranchName, req.getBranchName()));
        if (!CollectionUtils.isEmpty(companies)) throw new BusinessException("派出所已经存在，请重新填写派所名");
        Company company = companysMapper.selectList(new LambdaQueryWrapper<Company>().eq(Company::getCompanyId, getAccountInfo().getCompanyId()).orderByDesc(Company::getBranchId)).stream().findFirst().orElse(null);
        int insert1 = companysMapper.insert(company.setBranchId(String.valueOf(Integer.valueOf(company.getBranchId()) + 1)).setCompanyId(getAccountInfo().getCompanyId()).setBranchName(req.getBranchName()));
        if (insert1 == BigDecimal.ZERO.intValue()) throw new BusinessException("创建派出所失败！");
        int insert = mapper.insert(MapperManager.mapper(req, Police.class, p -> p.setPassword(MD5UtilNew.string2MD5(req.getPassword()))
                .setCreateTime(LocalDateTime.now()).setCompanyId(company.getId() + "").setType(getAccountInfo().setAreaId(getAccountInfo().getAreaId()).getType() + 1)));
        if (insert == BigDecimal.ZERO.intValue()) throw new BusinessException("创建派出所管理员账号异常！");
        return new InsertPoliceResp().setCount(insert + insert1);
    }

    /*
     * 查看详情
     * */
    @Override
    public DetailPoliceResp detail(DetailPoliceReq req) {
        Police police = mapper.selectById(req.getId());
        return MapperManager.mapper(police, DetailPoliceResp.class);
    }

    /**
     * 修改账号
     *
     * @param req 主键
     * @return 影响行数
     */
    @Override
    public UpdatePoliceResp update(UpdatePoliceReq req) {
        Integer id = req.getId();
        if (!StringUtil.isChinese(req.getUserName())) throw new BusinessException(UserConstant.NOT_CHINESE_USERNAME);
        Account account = accountsMapper.selectOne(new LambdaQueryWrapper<Account>().eq(Account::getAccount, req.getAccount()).eq(Account::getIsDelete, UserConstant.NOT_DELETE));
        return new UpdatePoliceResp().setCount(accountsMapper.updateById(MapperManager.mapper(req, Account.class, p -> p.setPassword(MD5UtilNew.string2MD5(req.getPassword()))
                .setUpdateTime(LocalDateTime.now()).setUpdateId(id))));

    }

    /*
     * 删除账号
     * */
    @Override
    @Transactional
    public DeleteAccountResp delete(DeleteAccountReq req) {
        return new DeleteAccountResp().setCount(accountsMapper.updateById(new Account().setId(req.getId())
                .setIsDelete(UserConstant.DELETE).setUpdateTime(LocalDateTime.now())));
    }
    /**
     * 派出所下拉查询（分局、派出所）
     *
     * @param req
     * @return
     */
    @Override
    public List<SelectPoliceLevelResp> selectLevel(SelectPoliceLevelReq req) {
        List<Company> companies = companysMapper.selectList(new LambdaQueryWrapper<Company>().eq(StringUtils.isNotEmpty(req.getCompanyId()), Company::getCompanyId, req.getCompanyId()));
        if (CollectionUtils.isEmpty(companies)) return Lists.newArrayList();
        ArrayList<Company> companyList = companies.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(Company::getCompanyId))), ArrayList::new));
        if (!req.getIsQueryBranch()) {
            return companyList.stream().map(p -> MapperManager.mapper(p, SelectPoliceLevelResp.class, q -> q.setId(p.getCompanyId()).setName(p.getCompanyName()))).collect(Collectors.toList());
        }
        return companyList.stream().map(p -> MapperManager.mapper(p, SelectPoliceLevelResp.class, q -> {
            List<Company> selectList = companysMapper.selectList(new LambdaQueryWrapper<Company>().eq(Company::getCompanyId, p.getCompanyId())
                    .eq(StringUtils.isNotEmpty(req.getBranchId()), Company::getId, req.getBranchId()));
            if (!CollectionUtils.isEmpty(selectList)) {
                q.setId(p.getCompanyId()).setName(p.getCompanyName()).setBranchList(selectList.stream().map(
                        t -> MapperManager.mapper(t, SelectPoliceLevelResp.BranchList.class, m -> m.setId(t.getBranchId()).setName(t.getBranchName()))
                ).collect(Collectors.toList()));
            }
        })).collect(Collectors.toList());
    }

}
