package com.metro.service.job;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.metro.entity.po.WarFruit;
import com.metro.mapper.PassreCordsMapper;
import com.metro.mapper.WarFruitMapper;
import com.metro.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @description:
 * @ClassName AdminDataShowTask
 * @Author WXG
 * @Date 2021/11/12 20:52
 */
@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
@EnableAsync
@Slf4j
public class WarFruitTodayTask {

    @Resource
    @Lazy
    private PassreCordsMapper passreCordsMapper;
    @Resource
    @Lazy
    private WarFruitMapper warFruitMapper;
    /*
    * 每天0点 民警的每日数据统计
    * */
    @Scheduled(fixedRate = 600000,zone = "Asia/Shanghai",initialDelay=120000)
    @Async
    public void dailyDataStatistics() {
        log.info("----------开始------民警每10分钟数据统计-------------");
        Date date = DateUtils.addHours(new Date(), -12);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        String passTime = DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss", calendar.getTime());
        List<WarFruit> list = warFruitMapper.updateData(passTime);
        if(CollectionUtil.isNotEmpty(list)){
            for (WarFruit warFruit : list) {
                this.warFruitEdit(warFruit);
            }
        }
        log.info("----------完成------民警每10分钟数据统计-------------");
    }

    private void warFruitEdit(WarFruit warFruit){
        LambdaQueryWrapper<WarFruit> warFruitWrapper = new LambdaQueryWrapper<>();
        warFruitWrapper.eq(WarFruit::getCardId,warFruit.getCardId());
        warFruitWrapper.eq(WarFruit::getDate,warFruit.getDate());
        WarFruit warFruit1 = warFruitMapper.selectOne(warFruitWrapper);
        if(warFruit1 != null){
            warFruit1.setNumber(warFruit.getNumber());
            warFruit1.setArrestNumber(warFruit.getArrestNumber());
            warFruit1.setDoubtNumber(warFruit.getDoubtNumber());
            warFruit1.setPassNumber(warFruit.getPassNumber());
            warFruit1.setExceptionNumber(warFruit.getExceptionNumber());
            warFruitMapper.updateById(warFruit1);
        }else {
            warFruitMapper.insert(warFruit);
        }
//        int type = 0;// 1-insert 0=update
//        if(warFruit == null){
//            warFruit = new WarFruit();
//            type = 1;
//        }
//        Integer arrestNumber = vo.getArrestNumber() == null? 0 : vo.getArrestNumber();
//        Integer doubtNumber = vo.getDoubtNumber() == null? 0 : vo.getDoubtNumber();
//        Integer passNumber = vo.getPassNumber() == null? 0 : vo.getPassNumber();
//        Integer exceptionNumber = vo.getExceptionNumber() == null? 0 : vo.getExceptionNumber();
//        warFruit.setCardId(vo.getCardId());
//        warFruit.setDate(vo.getDate());
//        warFruit.setArrestNumber(arrestNumber);
//        warFruit.setDoubtNumber(doubtNumber);
//        warFruit.setPassNumber(passNumber);
//        warFruit.setExceptionNumber(exceptionNumber);
//        warFruit.setNumber(arrestNumber + doubtNumber + passNumber + exceptionNumber);
//        if(type == 1 ){
//            warFruitMapper.insert(warFruit);
//        }else {
//            warFruitMapper.updateById(warFruit);
//        }
    }
}
