package com.metro.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class JsonUtil {
    private static final Logger logger = LoggerFactory.getLogger(JsonUtil.class);

    public static Map<String, Object> toMapFromJsonStr(String jsonStr) {
        if (StringUtil.isEmpty(jsonStr)) {
            return new HashMap();
        }
        Map<String, Object> argsMap;
        try {
            argsMap = (Map) com.alibaba.fastjson.JSONObject.parseObject(jsonStr, Map.class);
        } catch (Exception ex) {
            if (jsonStr.contains("\\")) {
                try {
                    jsonStr = jsonStr.replace("\\", "\\\\");
                    argsMap = (Map) com.alibaba.fastjson.JSONObject.parseObject(jsonStr, Map.class);
                } catch (Exception e) {
                    logger.error("JSON转Map出错，jsonStr=" + jsonStr, e);
                    argsMap = new HashMap();
                }
            } else {
                logger.error("JSON转Map出错，jsonStr=" + jsonStr, ex);
                argsMap = new HashMap();
            }
        }
        return argsMap;
    }

    public static String toJsonFromObject(Object obj) {
        if ((obj == null) || ("".equals(obj))) {
            return "";
        }
        try {
            return JSON.toJSONString(obj, new SerializerFeature[]{SerializerFeature.WriteNullStringAsEmpty});
        } catch (Exception e) {
            logger.error("Json转对象出错，obj=" + obj, e);
        }
        return null;
    }

    public static String toSqlJsonFromObject(Object obj) {
        String sqlStr = toJsonFromObject(obj);
        return sqlStr == null ? null : sqlStr.replaceAll("\\\\", "\\\\\\\\");
    }

    public static <T> T toBeanFromStr(String jsonStr, Class<T> c) {
        if (StringUtil.isEmpty(jsonStr)) {
            return null;
        }
        try {
            return (T) com.alibaba.fastjson.JSONObject.parseObject(jsonStr, c);
        } catch (Exception ex) {
            if (jsonStr.contains("\\")) {
                try {
                    jsonStr = jsonStr.replace("\\", "\\\\");
                    return (T) com.alibaba.fastjson.JSONObject.parseObject(jsonStr, c);
                } catch (Exception e) {
                    logger.error("JSON转对象出错，jsonStr=" + jsonStr, e);
                }
            } else {
                logger.error("JSON转对象出错，jsonStr=" + jsonStr, ex);
            }
        }
        return null;
    }

    public static <T> List<T> toListFromJsonStr(String jsonStr) {
        if (StringUtil.isEmpty(jsonStr)) {
            return new ArrayList();
        }
        List<T> argsMap;
        try {
            argsMap = (List) com.alibaba.fastjson.JSONObject.parseObject(jsonStr, List.class);
        } catch (Exception ex) {
            if (jsonStr.contains("\\")) {
                try {
                    jsonStr = jsonStr.replace("\\", "\\\\");
                    return (List) com.alibaba.fastjson.JSONObject.parseObject(jsonStr, List.class);
                } catch (Exception e) {
                    logger.error("JSON转list出错，jsonStr=" + jsonStr, e);
                }
            } else {
                logger.error("JSON转list出错，jsonStr=" + jsonStr, ex);
            }
            return new ArrayList();
        }
        return argsMap;
    }
}