package com.metro.utils;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.lang3.StringUtils;

/**
 * 分页查询相关参数。
 *
 * @author 永芬
 */
public class SearchPager {
    private int pageNumber = 1;// 查询页码

    private int pageSize = 10;// 每页大小

    // 查询表达式，注意：表达式中不能有空格
    private String expression;

    // 是否计算总数, 这里使用Boolean而不是boolean的原因:
    // 页面请求不一定每次都会有countTotal参数, 即countTotal参数有可能为null,
    // 而boolean类型只能是true或者false, 所以这里需要修改为Boolean类型
    private Boolean countTotal = null;

    private boolean autoSearch = false;// 是否自动查询

    /**
     * 获取查询页面，默认为第一页。
     */
    public int getPageNumber() {
        return pageNumber;
    }

    /**
     * 设置查询页码。
     */
    public void setPageNumber(int pageNumber) {
        this.pageNumber = pageNumber;
    }

    /**
     * 获取每页记录数，默认为每页10条记录。
     */
    public int getPageSize() {
        return pageSize;
    }

    public void setPageSize(int pageSize) {
        this.pageSize = pageSize;
    }

    /**
     * 获取查询条件表达式，主要用于自定义查询。
     */
    public String getExpression() {
        if (StringUtils.isNotBlank(expression)) {
            expression = StringUtils.replace(expression, " ", "+");
        }
        return expression;
    }

    /**
     * 设置查询条件表达式。
     */
    public void setExpression(String expression) {
        this.expression = expression;
    }

    /**
     * 判断是否计算总数，true表示计算，false表示不计算，默认为true。
     */
    public Boolean isCountTotal() {
        return countTotal;
    }

    /**
     * 设置是否记录总数，true表示计算，false表示不计算，默认为true。
     */
    public void setCountTotal(Boolean countTotal) {
        this.countTotal = countTotal;
    }

    /**
     * 判断是否自动查询，true表示自动查询，false表示不自动查询，默认为true。
     */
    public boolean isAutoSearch() {
        return autoSearch;
    }

    /**
     * 设置是否自动查询，true表示自动查询，false表示不自动查询，默认为true。
     */
    public void setAutoSearch(boolean autoSearch) {
        this.autoSearch = autoSearch;
    }

    public Page toPage() {
        return new Page<>(getPageNumber(), getPageSize());
    }
}
