package com.metro.utils;

import java.text.DecimalFormat;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <h3>StringUtil</h3>
 * <p>字符串工具类</p>
 *
 * @author : zouxucheng
 * @version : 1.0
 * @date : 2021-05-20 11:05
 **/
public class StringUtil {

    public static boolean isEmpty(String s) {
        return (s == null) || (s.trim().isEmpty()) || ("null".equalsIgnoreCase(s));
    }

    public static boolean isNotEmpty(String s) {
        return !isEmpty(s);
    }

    public static String toString(Object obj) {
        return obj == null ? null : obj.toString().trim();
    }

    public static String toStringNotNull(Object obj) {
        if (obj == null) {
            return "";
        }
        String s = obj.toString();
        return isEmpty(s) ? "" : s;
    }

    public static Integer toIntegerFromObj(Object obj) {
        String str = toStringNotNull(obj);
        return validate(str, "[+-]?\\d+") ? Integer.parseInt(str) : 0;
    }

    public static Long toLongFromObj(Object obj) {
        String str = toStringNotNull(obj);
        return validate(str, "[+-]?\\d+") ? Long.parseLong(str) : 0L;
    }

    public static Double toDoubleFromObj(Object obj) {
        String str = toStringNotNull(obj);
        return validate(str, "([+-]?\\d+(\\.\\d+)?)|(([-+]?\\d+\\.?\\d*)[Ee]{1}([-+]?\\d+))") ? Double.parseDouble(str) : 0.0D;
    }

    public static boolean validate(String arg, String regex) {
        return (StringUtil.isNotEmpty(arg)) && (arg.matches(regex));
    }

    public static String toDoubleStrFromObj(Object obj) {
        return toStrFromDouble(toDoubleFromObj(obj));
    }

    public static String toStrFromDouble(Double d) {
        return (d == null) || (d.doubleValue() == 0.0D) ? "" : String.format("%.2f", new Object[]{d});
    }

    public static String toStrFromList(List<Object> list, String separator) {
        if ((list == null) || (list.isEmpty())) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        for (Object s : list) {
            if (isNotEmpty(toStringNotNull(s))) {
                sb.append(s.toString());
                sb.append(separator);
            }
        }
        return sb.length() <= 0 ? "" : sb.substring(0, sb.length() - 1);
    }

    public static boolean isNumeric(String str) {
        if (isEmpty(str)) {
            return false;
        }
        int sz = str.length();
        for (int i = 0; i < sz; i++) {
            if (!Character.isDigit(str.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    public static boolean isChinese(String strName) {
        char[] ch = strName.toCharArray();
        for (int i = 0; i < ch.length; i++) {
            char c = ch[i];
            if (isChinese(c)) {
                return true;
            }
        }
        return false;
    }

    public static boolean isChinese(char c) {
        Character.UnicodeBlock ub = Character.UnicodeBlock.of(c);
        if ((ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS) || (ub == Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS) || (ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A) || (ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_B) || (ub == Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION) || (ub == Character.UnicodeBlock.HALFWIDTH_AND_FULLWIDTH_FORMS) || (ub == Character.UnicodeBlock.GENERAL_PUNCTUATION)) {
            return true;
        }
        return false;
    }

    public static String toSqlStrFromList(List<String> list) {
        if ((list == null) || (list.isEmpty())) {
            return "";
        }
        StringBuffer sb = new StringBuffer();
        for (String s : list) {
            if (isNotEmpty(s)) {
                sb.append("'");
                sb.append(convertSqlValue(s));
                sb.append("',");
            }
        }
        return sb.length() <= 0 ? "" : sb.substring(0, sb.length() - 1);
    }

    public static String convertSqlValue(String value) {
        if (null == value) {
            return null;
        }
        if (isEmpty(value)) {
            return "";
        }
        if (value.contains("'")) {
            value = value.replace("'", "''");
        }
        if (value.endsWith("\\")) {
            char[] arr = value.toCharArray();
            int count = 0;
            for (int i = arr.length; i > 0; i--) {
                if ('\\' != arr[(i - 1)]) {
                    break;
                }
                count++;
            }
            if ((count & 0x1) != 0) {
                value = value + "\\";
            }
        }
        return value;
    }

    public static String divition(double divisor, double dividend) {
        return divition(divisor, dividend, 2);
    }

    public static String divition(double divisor, double dividend, int num) {
        if (dividend == 0.0D) {
            return "";
        }
        return String.format("%." + num + "f", divisor / dividend);
    }

    public static String moneyFormat(String s) {
        if ((isEmpty(s)) || (!validate(s, "([+-]?\\d+(\\.\\d+)?)|(([-+]?\\d+\\.?\\d*)[Ee]{1}([-+]?\\d+))"))) {
            return "";
        }
        DecimalFormat df = new DecimalFormat("#,###.####");
        Double d = Double.parseDouble(s);
        return df.format(d);
    }

    public static String moneyFormat(String s, String unit) {
        if ((isEmpty(s)) || (!validate(s, "([+-]?\\d+(\\.\\d+)?)|(([-+]?\\d+\\.?\\d*)[Ee]{1}([-+]?\\d+))"))) {
            return "";
        }
        return moneyFormat(s).concat(unit);
    }

    public static String getNumFromStr(String s) {
        if (isEmpty(s)) {
            return "";
        }
        String regEx = "[^0-9.]";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(s);
        return m.replaceAll("").trim();
    }

    public static String getRandom() {
        String str = "";
        for (int i = 0;i<2;i++){
            str = str+ (char)(Math.random()*26+'A');
        }
        System.out.println(str);
        return str;
    }

}